import inquirer from "inquirer";
import { tsquery } from "@phenomnomnominal/tsquery";
import chalk from "chalk";
import path, {
  VIEW_URL,
  SRC_URL,
  TEMPLATE_URL,
  removeSlash,
} from "../utils/path.mjs";
import {
  barToHump,
  humpToBar,
  createComponentNameByUrl,
  createPageRootPath,
} from "../utils/name.mjs";
import ejs from "ejs";
import {
  createFile,
  changeFile,
  getFileCon,
  fileExist,
  createDir,
} from "../utils/file.mjs";

export const createPage = async () => {
  const { url } = await inquirer.prompt([
    {
      type: "String",
      name: "url",
      message: "请填写页面url",
      validate: (v) => {
        if (v) {
          return true;
        } else {
          console.warn(chalk.red("请填写页面url"));
          return false;
        }
      },
    },
  ]);

  const { componentName } = await inquirer.prompt([
    {
      type: "String",
      name: "componentName",
      message: "请填写页面组件名称[首字母大写，仅支持字母]",
      default: createComponentNameByUrl(url),
      validate: (v) => {
        const reg = /^[A-Z]\w*$/;

        // 校验：
        if (reg.test(v)) {
          return true;
        } else {
          console.warn(chalk.red("组件名称首字母大写，仅支持字母"));
          return false;
        }
      },
    },
  ]);

  const { filePath } = await inquirer.prompt([
    {
      type: "String",
      name: "filePath",
      message: "请填写页面组件代码文件路径[相对src/views下的路径，仅支持一级]",
      default: createPageRootPath(url),
    },
  ]);

  const { type } = await inquirer.prompt([
    {
      type: "list",
      message: "请选择页面类型",
      name: "type",
      choices: [
        {
          value: "empty",
          name: "空",
        },
        {
          value: "list",
          name: "列表",
        },
        {
          value: "form",
          name: "表单",
        },
      ],
    },
  ]);

  createPageFile(url, removeSlash(filePath), componentName, type);
};

const createPageFile = async (url, filePath, componentName, type) => {
  const configFileAbPath = path.join(VIEW_URL, filePath, "routers.tsx");
  const isFileExist = fileExist(configFileAbPath);

  if (!isFileExist) {
    // 创建页面跟文件
    createDir(path.join(VIEW_URL, filePath));

    const routerCode = await ejs.renderFile(TEMPLATE_URL + "/routers.ejs", {
      componentName,
      pageFileName: humpToBar(componentName),
      routerPath: /^\//.test(url) ? url : "/" + url,
    });

    createFile({
      path: path.join(VIEW_URL, filePath),
      name: "routers.tsx",
      con: routerCode,
    });

    const routerConfigPath = path.join(SRC_URL, "routers/index.tsx");
    // 修改公用router配置
    const oldCode = getFileCon(routerConfigPath);

    // 添加新的import
    const importSelect = "ImportDeclaration";
    const importLength = tsquery(oldCode, importSelect).length;
    let currImport = 0;
    const routeConfigName = barToHump(filePath);

    const codeAfterAddImport = tsquery.replace(
      oldCode,
      importSelect,
      (node) => {
        currImport += 1;

        if (importLength === currImport) {
          return `${node.getText()}
import ${routeConfigName} from "src/views/${filePath}/routers";`;
        } else {
          return node.getText();
        }
      }
    );

    const finalStr = tsquery.replace(
      codeAfterAddImport,
      "VariableDeclaration:has([name=routersConfig]) > ArrayLiteralExpression",
      (node) => {
        const arrStr = node.getText();

        if (arrStr.includes("\n")) {
          return arrStr.replace(
            "]",
            `  ...${routeConfigName},
]`
          );
        } else {
          return arrStr.replace("]", `, ...${routeConfigName}]`);
        }
      }
    );

    changeFile({
      url: routerConfigPath,
      con: finalStr,
    });
  }

  const componentFileName = humpToBar(
    componentName.replace(/^[A-Z]{1}/, (str) => str.toLowerCase())
  );
  const componentAbPath = path.join(VIEW_URL, filePath, componentFileName);

  // 创建页面组件
  createDir(componentAbPath);

  const componentClassName = `${filePath}-${componentFileName}`;

  let fileName = "/component.ejs";

  if (type === "list") {
    fileName = "/list.ejs";
  } else if (type === "form") {
    fileName = "/form.ejs";
  }

  const componentCode = await ejs.renderFile(TEMPLATE_URL + fileName, {
    className: componentClassName,
    componentName,
  });

  createFile({
    path: componentAbPath,
    name: "index.tsx",
    con: componentCode,
  });

  if (isFileExist) {
    const code = getFileCon(configFileAbPath);
    const importQuery = "VariableStatement:has(Identifier[name=lazy])";
    const importCount = tsquery(code, importQuery).length;
    let currRoute = 0;

    const afterAddImportCode = tsquery.replace(
      code,
      "VariableStatement:has(Identifier[name=lazy])",
      (node) => {
        currRoute += 1;

        if (currRoute === importCount) {
          return `${node.getText()}
const ${componentName} = lazy(() => import("./${componentFileName}"));`;
        } else {
          return node.getText();
        }
      }
    );

    const finalStr = tsquery.replace(
      afterAddImportCode,
      "VariableDeclaration:has(Identifier[name=routersConfig]) > ArrayLiteralExpression > ObjectLiteralExpression:last-child",
      (node) => {
        return `${node.getText()},
  {
    path: "${url}",
    element: <${componentName} />,
  }`;
      }
    );

    changeFile({
      url: configFileAbPath,
      con: finalStr,
    });
  }
};
